#extension GL_OES_EGL_image_external : require

/**
 * A simple shader that takes in a stereo texture, and adjusts the convergence.
 * It also zooms/crops to hide the reconvergence bands at the edges.
 **/

precision mediump float;

// A basic bitmap texture
uniform samplerExternalOES cameraTex;

// Texture coordinates, ranging from (0, 0) at the bottom-left of the screen,
//   to (1, 1) at the top-right of the screen.
varying vec2 vTextureCoord;

uniform float uConvergence;

void main() {
    // Convert coordinates such that (0, 0) is at the center of the 2 stereo images.
    // This is so that we zoom around the center of both images.
    vec2 viewCoord = vec2(mod(vTextureCoord.x * 2.0, 1.0), 1.0 - vTextureCoord.y);

    // Define zoom to be a function of convergence. More convergence requires cropping more.
    float zoom = 1.0 + 2.0 * abs(uConvergence);

    // Apply zoom.
    float m = 1.0 / zoom;
    float b = .5 * (1.0 - m);
    viewCoord = viewCoord * m + b;

    // Sample from the left and right images.
    vec2 sampleCoord;
    if (vTextureCoord.x >= .5) {
        sampleCoord = vec2(viewCoord.x / 2.0 + .5 - uConvergence / 2.0, viewCoord.y);
    } else {
        sampleCoord = vec2(viewCoord.x / 2.0 + uConvergence / 2.0, viewCoord.y);
    }

    gl_FragColor = texture2D(cameraTex, sampleCoord);
}
