#extension GL_OES_EGL_image_external : require

precision mediump float;

// A stereo camera texture, in horizontal side-by-side layout.
uniform sampler2D cameraTex;

// Texture coordinates, ranging from (0, 0) at the bottom-left of the screen,
//   to (1, 1) at the top-right of the screen.
varying vec2 vTextureCoord;

vec4 sampleFromLeft(vec2 coord) {
    return texture2D(cameraTex, vec2(coord.x / 2.0, coord.y));
}

vec4 sampleFromRight(vec2 coord) {
    return texture2D(cameraTex, vec2(coord.x / 2.0 + 0.5, coord.y));
}

/**
 * This shader program takes in a side-by-side stereo texture,
 *  and outputs a 4-view texture in 2x2 format.
 *
 *  Views 1 and 2 are duplicated from the left input,
 *  views 3 and 4 are duplicated from the right input.
 */
void main() {
    vec2 sampleCoord = mod(vTextureCoord, .5) * 2.0;

    if (vTextureCoord.x < .5 && vTextureCoord.y > .5) { // view 1 (upper left)
       gl_FragColor = sampleFromRight(sampleCoord);
    } else if (vTextureCoord.x > .5 && vTextureCoord.y > .5){ // view 2 (upper right)
       gl_FragColor = sampleFromRight(sampleCoord);
    } else if (vTextureCoord.x < .5 && vTextureCoord.y < .5) { // view 3 (bottom left)
       gl_FragColor = sampleFromLeft(sampleCoord);
    } else { // (vTextureCoord.x < .5 && vTextureCoord.y < .5) { // view 4 (bottom right)
       gl_FragColor = sampleFromLeft(sampleCoord);
    }
}
