#extension GL_OES_EGL_image_external : require

precision mediump float;

uniform sampler2D cameraTex;

uniform bool uDisparityTexIsBound;
uniform sampler2D disparityTex;

varying vec2 vTextureCoord;

vec4 sampleFromLeft(vec2 coord) {
    return texture2D(cameraTex, vec2(coord.x / 2.0, coord.y));
}

vec4 sampleFromRight(vec2 coord) {
    return texture2D(cameraTex, vec2(coord.x / 2.0 + 0.5, coord.y));
}

/**
 * This shader program takes in a side-by-side stereo texture and disparity map texture
 *  and outputs a side-by-side stereo texture, with the disparity cancelled out.
 *
 * If the disparity map is correct, the left and right output images are identical.
 * We calculate the error (the "residual disparity") and apply it to the original disparity.
 */
void main() {

    float y = 1.0 - vTextureCoord.y;

    float disparity;
    if (uDisparityTexIsBound) {
        vec2 disparityCoord = vec2(mod(vTextureCoord.x * 2.0, 1.0), y);

        disparity = texture2D(disparityTex, disparityCoord).r;

        // Map from [0,1] to percentage disparity [-.05, .05]
        disparity = (disparity - .5) * .1;

        // We adjust each view by 1/2 or -1/2.
        disparity /= 2.0;
    } else {
        // If no disparity texture is specified, set disparity to 0.
        disparity = 0.0;
    }

    if (vTextureCoord.x < .5) {
        vec2 coord = vec2(vTextureCoord.x * 2.0 + disparity, y);
        gl_FragColor = sampleFromLeft(coord);
    } else {
        vec2 coord = vec2(vTextureCoord.x * 2.0 - 1.0 - disparity, y);
        gl_FragColor = sampleFromRight(coord);
    }
}
